!====================================================================================================
!
! General Boundary Conditions: enforce physical BC and compute Eqns with Boundary Scheme
!                              Both steady and unsteady BC can be generated (i.e. on vector q or rhs)
!     
!====================================================================================================

subroutine phyBCNAME(param_float,indbc,hlo,nrk,neq,indvars,nx,ny,nz,sizeblck,q,q1,q2,rhs)

  implicit none

#include "dtypes.h"
#include "param_fort.h"

  
  integer :: i,j,k
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

  real(wp), intent(in)    :: param_float(*)

  integer, intent(in) :: nx,ny,nz,hlo,nrk,neq
  integer, intent(in) :: indvars(1:neq)
  integer, intent(in) :: indbc(6)
  integer, intent(in) :: sizeblck(3)

  real(wp),  dimension(1:3) :: rk1 = (/ 2.0_wp/ 3.0_wp, &
                                        5.0_wp/12.0_wp, &
                                        3.0_wp/ 5.0_wp /)
  real(wp),  dimension(1:3) :: rk2 = (/ 1.0_wp/ 4.0_wp, &
                                        3.0_wp/20.0_wp, &
                                        3.0_wp/ 5.0_wp /)

  real(wp) :: one_over_rho
  
#include "includeRHS_globVar_rk3.f90"

!PHYBCLOCVAR_q

!PHYBCLOCVAR_rhs
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: q1,q2,a,nx,ny,nz,rhs,nrk
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)

      
size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)


!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2)  
  do bk=indbc(5),indbc(6),size_bk  
    do bj=indbc(3),indbc(4),size_bj 
      do bi=indbc(1),indbc(2),size_bi 
    
idloop(6) = min( bk+size_bk, indbc(6)+1)-1
idloop(4) = min( bj+size_bj, indbc(4)+1)-1
idloop(2) = min( bi+size_bi, indbc(2)+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 

!PHYBCLOOPS_q

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO


!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2)  
  do bk=indbc(5),indbc(6),size_bk  
    do bj=indbc(3),indbc(4),size_bj 
      do bi=indbc(1),indbc(2),size_bi 
    
   
idloop(6) = min( bk+size_bk, indbc(6)+1)-1
idloop(4) = min( bj+size_bj, indbc(4)+1)-1
idloop(2) = min( bi+size_bi, indbc(2)+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 

!PHYBCLOOPS_rhs

if (nrk == 1) then

!PHYBCLOOPS_P2CONS

endif  

!PHYBCLOOPS_RK3step

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
 
!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2)       
  do bk=indbc(5),indbc(6),size_bk  
    do bj=indbc(3),indbc(4),size_bj 
      do bi=indbc(1),indbc(2),size_bi 
   
idloop(6) = min( bk+size_bk, indbc(6)+1)-1
idloop(4) = min( bj+size_bj, indbc(4)+1)-1
idloop(2) = min( bi+size_bi, indbc(2)+1)-1
 
 idloop(5) = bk
 idloop(3) = bj
 idloop(1) = bi 

!PHYBCLOOPS_RK3update

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO NOWAIT

!$OMP END PARALLEL

 end subroutine phyBCNAME
