  

  subroutine time(intparam,fltparam,fltdata)

  implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
  real(wp), intent(inout) ::  fltdata(*)
  integer,  intent(in)    :: intparam(*)
  
  call rk3_stepk(fltparam                                            ,&
                 intparam(iadrHLO), intparam(iadrNRK)                ,&
                 intparam(iadrNVAR)                                  ,&
                 intparam(iadrVARS)                                  ,&
                 intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                 intparam(iadrCacheBLCK)                             ,&
                 fltdata(1)                                          ,& ! q
                 fltdata(1+intparam(iadrNDIMTOT)  )                  ,& ! q1
                 fltdata(1+intparam(iadrNDIMTOT)*2)                  ,& ! q2 
                 fltdata(1+intparam(iadrNDIMTOT)*3)                   ) ! rhs        


  end subroutine time  



 subroutine rk3_stepk(fltparam,hlo,nrk,neq,indvars,nx,ny,nz,sizeblck,q,q1,q2,rhs)

  implicit none

#include "dtypes.h"
#include "param_fort.h"

  
  integer :: i,j,k
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

  real(wp), intent(in)    :: fltparam(*)

  integer, intent(in) :: nx,ny,nz,hlo,nrk,neq
  integer, intent(in) :: indvars(1:neq)
  integer, intent(in) :: sizeblck(3)

  real(wp),  dimension(1:3) :: rk1 = (/ 2.0_wp/ 3.0_wp, &
                                        5.0_wp/12.0_wp, &
                                        3.0_wp/ 5.0_wp /)
  real(wp),  dimension(1:3) :: rk2 = (/ 1.0_wp/ 4.0_wp, &
                                        3.0_wp/20.0_wp, &
                                        3.0_wp/ 5.0_wp /)

  real(wp) :: one_over_rho
  
#include "includeRHS_globVar_rk3.f90"

  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: q1,q2,a,nx,ny,nz,rhs,nrk
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)

      
  idarray(1) = 1 -hlo
  idarray(2) = nx+hlo

  idarray(3) = 1 -hlo
  idarray(4) = ny+hlo

  idarray(5) = 1 -hlo
  idarray(6) = nz+hlo

size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2)  
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
idloop(6) = min( bk+size_bk, nz+1)-1
idloop(4) = min( bj+size_bj, ny+1)-1
idloop(2) = min( bi+size_bi, nx+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 

call cmprhs(fltparam,indvars,idloop,idarray,neq,sizeblck,q,rhs)

if (nrk == 1) then

#include "LOOP_BEGIN"

#include "primitive_to_conservative.f90"

#include "LOOP_END"

endif  

#include "LOOP_BEGIN"

#include "includeRK3.f90"       
     
#include "LOOP_END"

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
 
!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2)       
   do bk=1,nz,size_bk  
     do bj=1,ny,size_bj 
       do bi=1,nx,size_bi 
     
 idloop(6) = min( bk+size_bk, nz+1)-1
 idloop(4) = min( bj+size_bj, ny+1)-1
 idloop(2) = min( bi+size_bi, nx+1)-1
 
 idloop(5) = bk
 idloop(3) = bj
 idloop(1) = bi 

#include "LOOP_BEGIN"

#include "includeRK3update.f90"       
     
#include "LOOP_END"

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO NOWAIT

!$OMP END PARALLEL

 end subroutine rk3_stepk


subroutine filter(dir,intparam,fltparam,fltdata)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

real(wp), intent(in)    :: fltparam(*)
real(wp), intent(inout) ::  fltdata(*)
integer,  intent(in)    :: intparam(*)

integer, intent(in)   :: dir

!f2py  intent(in)   :: intparam,fltparam,dir
!f2py intent(inout) :: fltdata    

 if     (dir == 1) then

    call flt_x(fltparam                                            ,&
               intparam(iadrHLO)                                   ,&
               intparam(iadrNVAR)                                  ,&
               intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
               intparam(iadrCacheBLCK)                             ,&               
               fltdata(1)                                          ,&
               fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array
    
 elseif (dir == 2)then

    call flt_y(fltparam                                            ,&
               intparam(iadrHLO)                                   ,&
               intparam(iadrNVAR)                                  ,&
               intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
               intparam(iadrCacheBLCK)                             ,&                              
               fltdata(1)                                          ,&
               fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array

 elseif (dir == 3)then


    call flt_z(fltparam                                            ,&
               intparam(iadrHLO)                                   ,&
               intparam(iadrNVAR)                                  ,&
               intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
               intparam(iadrCacheBLCK)                             ,&                              
               fltdata(1)                                          ,&
               fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array

 endif

 end subroutine filter


subroutine flt_x(fltparam,hlo,neq,nx,ny,nz,sizeblck,q,q2)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
! LOCAL VARIABLES

  integer, intent(in) :: nx,ny,nz,hlo,neq  
  integer, intent(in) :: sizeblck(3)

  integer :: i,j,k  
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

#include "includeRHS_globVar_filter.f90"
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: nx,ny,nz,hlo,neq,indvars,fltparam
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)

  idarray(1) = 1 -hlo
  idarray(2) = nx+hlo

  idarray(3) = 1 -hlo
  idarray(4) = ny+hlo

  idarray(5) = 1 -hlo
  idarray(6) = nz+hlo

size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
idloop(6) = min( bk+size_bk, nz+1)-1
idloop(4) = min( bj+size_bj, ny+1)-1
idloop(2) = min( bi+size_bi, nx+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 

#include "LOOP_BEGIN"

#include "Filter_x"    
     
#include "LOOP_END"

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
 
!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
idloop(6) = min( bk+size_bk, nz+1)-1
idloop(4) = min( bj+size_bj, ny+1)-1
idloop(2) = min( bi+size_bi, nx+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 

#include "LOOP_BEGIN"  
#include "updateFilter_x"   
#include "LOOP_END"
  
     enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
!$OMP END PARALLEL

 end subroutine flt_x

subroutine flt_y(fltparam,hlo,neq,nx,ny,nz,sizeblck,q,q2)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
! LOCAL VARIABLES

  integer, intent(in) :: nx,ny,nz,hlo,neq  
  integer, intent(in) :: sizeblck(3)

  integer :: i,j,k  
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

#include "includeRHS_globVar_filter.f90"
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: nx,ny,nz,hlo,neq,indvars,fltparam
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)

 idarray(1) = 1 -hlo
 idarray(2) = nx+hlo

 idarray(3) = 1 -hlo
 idarray(4) = ny+hlo

 idarray(5) = 1 -hlo
 idarray(6) = nz+hlo


size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
  idloop(6) = min( bk+size_bk, nz+1)-1
  idloop(4) = min( bj+size_bj, ny+1)-1
  idloop(2) = min( bi+size_bi, nx+1)-1

  idloop(5) = bk
  idloop(3) = bj
  idloop(1) = bi

#include "LOOP_BEGIN"

#include "Filter_y"    
     
#include "LOOP_END"

     enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
  idloop(6) = min( bk+size_bk, nz+1)-1
  idloop(4) = min( bj+size_bj, ny+1)-1
  idloop(2) = min( bi+size_bi, nx+1)-1

  idloop(5) = bk
  idloop(3) = bj
  idloop(1) = bi

#include "LOOP_BEGIN"  
#include "updateFilter_y"   
#include "LOOP_END"
  
     enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
!$OMP END PARALLEL


 end subroutine flt_y


 subroutine flt_z(fltparam,hlo,neq,nx,ny,nz,sizeblck,q,q2)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
! LOCAL VARIABLES

  integer, intent(in) :: nx,ny,nz,hlo,neq  
  integer, intent(in) :: sizeblck(3)

  integer :: i,j,k  
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

#include "includeRHS_globVar_filter.f90"
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: nx,ny,nz,hlo,neq,indvars,fltparam
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)


  idarray(1) = 1 -hlo
  idarray(2) = nx+hlo

  idarray(3) = 1 -hlo
  idarray(4) = ny+hlo

  idarray(5) = 1 -hlo
  idarray(6) = nz+hlo

size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
idloop(6) = min( bk+size_bk, nz+1)-1
idloop(4) = min( bj+size_bj, ny+1)-1
idloop(2) = min( bi+size_bi, nx+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 


#include "LOOP_BEGIN"

#include "Filter_z"    
     
#include "LOOP_END"


     enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
  idloop(6) = min( bk+size_bk, nz+1)-1
  idloop(4) = min( bj+size_bj, ny+1)-1
  idloop(2) = min( bi+size_bi, nx+1)-1

  idloop(5) = bk
  idloop(3) = bj
  idloop(1) = bi

#include "LOOP_BEGIN"  
#include "updateFilter_z"   
#include "LOOP_END"
  
     enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
!$OMP END PARALLEL

 end subroutine flt_z


subroutine bc(dir,intparam,fltparam,fltdata)!fltparam,hlo,nrk,neq,indvars,nx,ny,nz,q,q1,q2,rhs)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

real(wp), intent(in)    :: fltparam(*)
real(wp), intent(inout) ::  fltdata(*)
integer,  intent(in)    :: intparam(*)

integer, intent(in)   :: dir

!f2py  intent(in)   :: intparam,fltparam,dir
!f2py intent(inout) :: fltdata    

 if     (dir == 1) then

    call periodic_x(fltparam                                            ,&
                    intparam(iadrHLO)                                   ,&
                    intparam(iadrNVAR)                                  ,&
                    intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                    intparam(iadrCacheBLCK)                             ,&               
                    fltdata(1)                                          ,&
                    fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array
    
 elseif (dir == 2)then

    call periodic_y(fltparam                                            ,&
                    intparam(iadrHLO)                                   ,&
                    intparam(iadrNVAR)                                  ,&
                    intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                    intparam(iadrCacheBLCK)                             ,&                              
                    fltdata(1)                                          ,&
                    fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array

 elseif (dir == 3)then


    call periodic_z(fltparam                                            ,&
                    intparam(iadrHLO)                                   ,&
                    intparam(iadrNVAR)                                  ,&
                    intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                    intparam(iadrCacheBLCK)                             ,&                              
                    fltdata(1)                                          ,&
                    fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array

 elseif (dir == 0 )then 

  call periodic_x(fltparam                                            ,&
                    intparam(iadrHLO)                                   ,&
                    intparam(iadrNVAR)                                  ,&
                    intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                    intparam(iadrCacheBLCK)                             ,&               
                    fltdata(1)                                          ,&
                    fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array
  call periodic_y(fltparam                                            ,&
                    intparam(iadrHLO)                                   ,&
                    intparam(iadrNVAR)                                  ,&
                    intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                    intparam(iadrCacheBLCK)                             ,&                              
                    fltdata(1)                                          ,&
                    fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array
  call periodic_z(fltparam                                            ,&
                    intparam(iadrHLO)                                   ,&
                    intparam(iadrNVAR)                                  ,&
                    intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                    intparam(iadrCacheBLCK)                             ,&                              
                    fltdata(1)                                          ,&
                    fltdata(1+intparam(iadrNDIMTOT)*2)                   ) ! q2 is used for filter working array

    
 endif



 end subroutine bc

subroutine periodic_x(fltparam,hlo,neq,nx,ny,nz,sizeblck,q,q2)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
! LOCAL VARIABLES

  integer, intent(in) :: nx,ny,nz,hlo,neq  
  integer, intent(in) :: sizeblck(3)

  integer :: i,j,k  
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

#include "includeRHS_globVar_filter.f90"
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: nx,ny,nz,hlo,neq,indvars,fltparam
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)


  idarray(1) = 1 -hlo
  idarray(2) = nx+hlo

  idarray(3) = 1 -hlo
  idarray(4) = ny+hlo

  idarray(5) = 1 -hlo
  idarray(6) = nz+hlo

size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
    
idloop(6) = min( bk+size_bk, nz +1)-1
idloop(4) = min( bj+size_bj, ny +1)-1
idloop(2) = 0

idloop(5) = bk
idloop(3) = bj
idloop(1) = 0 

#include "LOOP_BEGIN"

#include "periodic_x"    
     
#include "LOOP_END"

  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
!$OMP END PARALLEL
 
 end subroutine periodic_x

subroutine periodic_y(fltparam,hlo,neq,nx,ny,nz,sizeblck,q,q2)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
! LOCAL VARIABLES

  integer, intent(in) :: nx,ny,nz,hlo,neq  
  integer, intent(in) :: sizeblck(3)

  integer :: i,j,k  
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

#include "includeRHS_globVar_filter.f90"
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: nx,ny,nz,hlo,neq,indvars,fltparam
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)

 idarray(1) = 1 -hlo
 idarray(2) = nx+hlo

 idarray(3) = 1 -hlo
 idarray(4) = ny+hlo

 idarray(5) = 1 -hlo
 idarray(6) = nz+hlo


size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2) 
  do bk=1,nz,size_bk      
      do bi=1-hlo,nx+hlo,size_bi 
    
  idloop(6) = min( bk+size_bk, nz+1)-1
  idloop(4) = 0
  idloop(2) = min( bi+size_bi, nx+hlo+1)-1

  idloop(5) = bk
  idloop(3) = 0
  idloop(1) = bi

#include "LOOP_BEGIN"

#include "periodic_y"    
     
#include "LOOP_END"

     enddo ! END cache blocking i
enddo ! END cache blocking k
!$OMP END DO
!$OMP END PARALLEL


 end subroutine periodic_y


 subroutine periodic_z(fltparam,hlo,neq,nx,ny,nz,sizeblck,q,q2)

implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
! LOCAL VARIABLES

  integer, intent(in) :: nx,ny,nz,hlo,neq  
  integer, intent(in) :: sizeblck(3)

  integer :: i,j,k  
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

#include "includeRHS_globVar_filter.f90"
  
  integer :: idloop(6),idarray(6)


!f2py intent(in)    :: nx,ny,nz,hlo,neq,indvars,fltparam
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)


  idarray(1) = 1 -hlo
  idarray(2) = nx+hlo

  idarray(3) = 1 -hlo
  idarray(4) = ny+hlo

  idarray(5) = 1 -hlo
  idarray(6) = nz+hlo

size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(GUIDED,4) COLLAPSE(2)  
    do bj=1-hlo,ny+hlo,size_bj 
      do bi=1-hlo,nx+hlo,size_bi 
    
idloop(6) = 0
idloop(4) = min( bj+size_bj, ny+hlo+1)-1
idloop(2) = min( bi+size_bi, nx+hlo+1)-1

idloop(5) = 0
idloop(3) = bj
idloop(1) = bi 


#include "LOOP_BEGIN"

#include "periodic_z"    
     
#include "LOOP_END"


     enddo ! END cache blocking i
  enddo ! END cache blocking j
!$OMP END DO
!$OMP END PARALLEL

 end subroutine periodic_z

subroutine pack(buf,f,ibeg,iend,jbeg,jend,kbeg,kend,sizex,sizey,sizez,sizenv)

implicit none

#include "dtypes.h"

real(wp), intent(inout) :: buf(*),f(*)
integer,intent(in)      :: ibeg,iend,jbeg,jend,kbeg,kend,sizex,sizey,sizez,sizenv

integer :: sx,sy,sz,i,j,k,n,indbuf,indf

sx = iend - ibeg 
sy = jend - jbeg 
sz = kend - kbeg 


!$OMP PARALLEL DEFAULT(SHARED) private(indbuf,indf)

!$OMP DO SCHEDULE(STATIC) COLLAPSE(3)
do n=1,sizenv
 do k=kbeg,kend-1
    do j=jbeg,jend-1 
!$omp simd 
      do i=ibeg,iend-1 
        indbuf = i - ibeg + 1 + (j-jbeg)*sx + (k-kbeg)*sx*sy + (n-1)*sx*sy*sz
        indf   = i        + 1 + j*sizex     + k*sizex*sizey  + (n-1)*sizex*sizey*sizez
        buf(indbuf) = f(indf)
      enddo
    enddo
 enddo
enddo 
!$OMP END DO NOWAIT
!$OMP END PARALLEL

end subroutine pack  


subroutine unpack(buf,f,ibeg,iend,jbeg,jend,kbeg,kend,sizex,sizey,sizez,sizenv)

implicit none

#include "dtypes.h"

real(wp), intent(inout) :: buf(*),f(*)
integer,intent(in)      :: ibeg,iend,jbeg,jend,kbeg,kend,sizex,sizey,sizez,sizenv

integer :: sx,sy,sz,i,j,k,n,indbuf,indf

sx = iend - ibeg 
sy = jend - jbeg 
sz = kend - kbeg 


!$OMP PARALLEL DEFAULT(SHARED) private(indbuf,indf)

!$OMP DO SCHEDULE(STATIC) COLLAPSE(3)
do n=1,sizenv
 do k=kbeg,kend-1
    do j=jbeg,jend-1 
!$omp simd       
      do i=ibeg,iend-1 
        indbuf = i - ibeg + 1 + (j-jbeg)*sx + (k-kbeg)*sx*sy + (n-1)*sx*sy*sz
        indf   = i        + 1 + j*sizex     + k*sizex*sizey  + (n-1)*sizex*sizey*sizez
        f(indf) = buf(indbuf)
      enddo
    enddo
 enddo
enddo 
!$OMP END DO NOWAIT
!$OMP END PARALLEL

end subroutine unpack  

 subroutine init(intparam,fltparam,fltdata)

  implicit none  

#include "dtypes.h"
#include "param_fort.h" 

  real(wp), intent(in)    :: fltparam(*)
  real(wp), intent(inout) ::  fltdata(*)
  integer,  intent(in)    :: intparam(*)

  call init_numa(fltparam                                            ,&
                 intparam(iadrHLO), intparam(iadrNRK)                ,&
                 intparam(iadrNVAR)                                  ,&
                 intparam(iadrVARS)                                  ,&
                 intparam(iadrNX),intparam(iadrNY) , intparam(iadrNZ),&
                 intparam(iadrCacheBLCK)                             ,&
                 fltdata(1)                                          ,& ! q
                 fltdata(1+intparam(iadrNDIMTOT)  )                  ,& ! q1
                 fltdata(1+intparam(iadrNDIMTOT)*2)                  ,& ! q2 
                 fltdata(1+intparam(iadrNDIMTOT)*3)                   ) ! rhs        


  end subroutine init

subroutine init_numa(fltparam,hlo,nrk,neq,indvars,nx,ny,nz,sizeblck,q,q1,q2,rhs)

  implicit none

#include "dtypes.h"
#include "param_fort.h"

  
  integer :: i,j,k
  integer :: bi,bj,bk
  integer :: biend,bjend,bkend
  integer :: size_bi,size_bj,size_bk

  real(wp), intent(in)    :: fltparam(*)

  integer, intent(in) :: nx,ny,nz,hlo,nrk,neq
  integer, intent(in) :: indvars(1:neq)
  integer, intent(in) :: sizeblck(3)
  
#include "includeRHS_globVar_rk3.f90"

  
  integer :: idloop(6),idarray(6)

!f2py intent(in)    :: q1,q2,a,nx,ny,nz,rhs,nrk
!f2py intent(inout) :: q

!$OMP PARALLEL DEFAULT(SHARED) private(idloop)

  idarray(1) = 1 -hlo
  idarray(2) = nx+hlo

  idarray(3) = 1 -hlo
  idarray(4) = ny+hlo

  idarray(5) = 1 -hlo
  idarray(6) = nz+hlo

size_bk = sizeblck(3)
size_bj = sizeblck(2)
size_bi = sizeblck(1)

!$OMP DO SCHEDULE(STATIC) 
  do bk=1,nz,size_bk  
    do bj=1,ny,size_bj 
      do bi=1,nx,size_bi 
    
idloop(6) = min( bk+size_bk, nz+1)-1
idloop(4) = min( bj+size_bj, ny+1)-1
idloop(2) = min( bi+size_bi, nx+1)-1

idloop(5) = bk
idloop(3) = bj
idloop(1) = bi 


#include "LOOP_BEGIN"

#include "init_numa.f90"

#include "LOOP_END"

    enddo ! END cache blocking i
  enddo ! END cache blocking j
enddo ! END cache blocking k
!$OMP END DO
!$OMP END PARALLEL

end subroutine init_numa
