
dim = 3 # 1 :1D, 2 : 2D, 3 : 3D

# Coefficitents needed in the pdes

coefficients = {'visc'       : 1, 
                'kappa'      : 2,  
                'gamma_m1'   : 3,
                'CvInv'      : 4}

varname      = {'rho' : 1,
                  'u' : 2, 
                  'v' : 3, 
                  'w' : 4,
                  'et': 5}

consvar      = [2,3,4,5]            

varloc       = {'p': 'gamma_m1*rho*(e)',
                'e': 'et-0.5_wp*(u*u+v*v+w*w)',
                'T': 'CvInv*(e)'}

# These names are for the comments in the src

rhsname = {'rho' : 'd(rho)/dt'  
          ,  'u' : 'd(rho u)/dt',
             'v' : 'd(rho v)/dt', 
             'w' : 'd(rho w)/dt', 
             'et': 'd(rho et)/dt'}
       

# These names are for the variables in the fortran src

vname = {'rho': {'x':'Flx_rho ','y':'Fly_rho ','z':'Flz_rho '},
         'u'  : {'x':'Flx_rhou','y':'Fly_rhou','z':'Flz_rhou'},
         'v'  : {'x':'Flx_rhov','y':'Fly_rhov','z':'Flz_rhov'},
         'w'  : {'x':'Flx_rhow','y':'Fly_rhow','z':'Flz_rhow'},
         'et' : {'x':'Flx_et  ','y':'Fly_et  ','z':'Flz_et  '}}


locname_dif = {'rho': 'dif_rho',
               'u'  : 'dif_rhou',
               'v'  : 'dif_rhov',
               'w'  : 'dif_rhow',
               'et' : 'dif_et  '}

locname_conv = {'rho': 'conv_rho',
               'u'  : 'conv_rhou',
               'v'  : 'conv_rhov',
               'w'  : 'conv_rhow',
               'et' : 'conv_et  '}

# Start specifying the PDEs

# IMPORTANT SYNTAX RULE :
# put a space after derivative symbol []_1* 


divops = '[u]_1x + [v]_1y + [w]_1z'

# Euler RHS        

Fx = {'rho' : 'rho*u         ',
      'u'   : 'rho*u*u  + p  ',
      'v'   : 'rho*u*v       ',
      'w'   : 'rho*u*w       ',
      'et'  : '(rho*et + p)*u '}

Fy = {'rho' : 'rho*v         ',
      'u'   : 'rho*v*u       ', 
      'v'   : 'rho*v*v  + p  ', 
      'w'   : 'rho*v*w       ', 
      'et'  : '(rho*et + p)*v '} 
                             # 
       # 
Fz = {'rho' : 'rho*w         ',
      'u'   : 'rho*w*u       ',
      'v'   : 'rho*w*v       ',
      'w'   : 'rho*w*w  + p  ',
      'et'  : '(rho*et + p)*w '}

Flxconv = { 'x': Fx , 'y': Fy, 'z': Fz }

Src_conv = {'rho' : '[ '+Fx['rho']+' ]_1x' + ' + ' + '[ '+Fy['rho']+' ]_1y' + ' + ' + '[ '+Fz['rho']+' ]_1z ',
            'u'   : '[ '+Fx['u']  +' ]_1x' + ' + ' + '[ '+Fy['u']  +' ]_1y' + ' + ' + '[ '+Fz['u']  +' ]_1z ',
            'v'   : '[ '+Fx['v']  +' ]_1x' + ' + ' + '[ '+Fy['v']  +' ]_1y' + ' + ' + '[ '+Fz['v']  +' ]_1z ',
            'w'   : '[ '+Fx['w']  +' ]_1x' + ' + ' + '[ '+Fy['w']  +' ]_1y' + ' + ' + '[ '+Fz['w']  +' ]_1z ',
            'et' : ' [ '+Fx['et'] +' ]_1x' + ' + ' + '[ '+Fy['et'] +' ]_1y' + ' + ' + '[ '+Fz['et'] +' ]_1z '}

# Navier-Stokes Diffusive termes only
ddivops = '{u}_1x + {v}_1y + {w}_1z'

Fx = {'u'   : ' - visc *( 2.0_wp * {u}_1x - 2.0_wp/3.0_wp * ( '+ ddivops +'  ) )',
      'v'   : ' - visc *( {u}_1y + {v}_1x )', 
      'w'   : ' - visc *( {u}_1z + {w}_1x )',
      
      'et'  : ' - kappa*( {T}_1x ) '
              ' - u*(visc *( 2.0_wp * {u}_1x - 2.0_wp/3.0_wp * ( '+ ddivops +'  )))'
              ' - v*(visc *( {u}_1y + {v}_1x ))'
              ' - w*(visc *( {u}_1z + {w}_1x ))'}

Fy = {'u'   : ' - visc *( {u}_1y + {v}_1x )  ',
      'v'   : ' - visc *( 2.0_wp * {v}_1y - 2.0_wp/3.0_wp * ( '+ ddivops +'  ) )', 
      'w'   : ' - visc *( {v}_1z + {w}_1y )  ',
      'et'  : ' - kappa*( {T}_1y )'
              ' - u*(visc *( {u}_1y + {v}_1x ))'
              ' - v*(visc *( 2.0_wp * {v}_1y - 2.0_wp/3.0_wp * ( '+ ddivops +'  )))'
              ' - w*(visc *( {v}_1z + {w}_1y ))'}
       
Fz = {'u'   : ' - visc *( {u}_1z + {w}_1x )',
      'v'   : ' - visc *( {v}_1z + {w}_1y )', 
      'w'   : ' - visc *( 2.0_wp * {w}_1z - 2.0_wp/3.0_wp * ( '+ ddivops +'  ) )',
      'et'  : ' - kappa*( {T}_1z )'
              ' - u*(visc *( {u}_1z + {w}_1x ))'
              ' - v*(visc *( {v}_1z + {w}_1y ))'
              ' - w*(visc *( 2.0_wp * {w}_1z - 2.0_wp/3.0_wp * ( '+ ddivops +'  )))'}

Flxdif = { 'x': Fx , 'y': Fy, 'z': Fz }

Src_dif  = {'u'   : '[ '+Fx['u']  +' ]_1x' + ' + ' + '[ '+Fy['u']  +' ]_1y' + ' + ' + '[ '+Fz['u']  +' ]_1z ',
            'v'   : '[ '+Fx['v']  +' ]_1x' + ' + ' + '[ '+Fy['v']  +' ]_1y' + ' + ' + '[ '+Fz['v']  +' ]_1z ',
            'w'   : '[ '+Fx['w']  +' ]_1x' + ' + ' + '[ '+Fy['w']  +' ]_1y' + ' + ' + '[ '+Fz['w']  +' ]_1z ',
            'et' : ' [ '+Fx['et'] +' ]_1x' + ' + ' + '[ '+Fy['et'] +' ]_1y' + ' + ' + '[ '+Fz['et'] +' ]_1z '}


ddivops = '{u}_1x + {v}_1y + {w}_1z'

# Navier-Stokes Diffusive terms only (Laplacian form)

SLaplace = {'u'   : '- visc* ( [u]_2xx + [u]_2yy + [u]_2zz ) - visc*0.3333333333333333_wp*( [ '+ ddivops+' ]_1x )  ',
     'v'   : '- visc* ( [v]_2xx + [v]_2yy + [v]_2zz ) - visc*0.3333333333333333_wp*( [ '+ ddivops+' ]_1y )  ',
     'w'   : '- visc* ( [w]_2xx + [w]_2yy + [w]_2zz ) - visc*0.3333333333333333_wp*( [ '+ ddivops+' ]_1z )  ',
     'et'  : '- visc* ( ( [u]_2xx + [u]_2yy + [u]_2zz )*u '
             ' +        ( [v]_2xx + [v]_2yy + [v]_2zz )*v '
             ' +        ( [w]_2xx + [w]_2yy + [w]_2zz )*w )'
             '- visc*0.3333333333333333_wp*('
             '  [ '+ ddivops+' ]_1x * u '
             '+ [ '+ ddivops+' ]_1y * v '
             '+ [ '+ ddivops+' ]_1z * w )'
             '- visc* ('
             ' ( 2.0_wp * [u]_1x * [u]_1x ) '
             '+( [u]_1y + [v]_1x ) * [v]_1x '
             '+( [u]_1z + [w]_1x ) * [w]_1x '
             '+( 2.0_wp * [v]_1y * [v]_1y ) '
             '+( [v]_1x + [u]_1y ) * [u]_1y '
             '+( [v]_1z + [w]_1y ) * [w]_1y '
             '+( 2.0_wp * [w]_1z * [w]_1z ) '
             '+( [w]_1x + [u]_1z ) * [u]_1z '
             '+( [w]_1y + [v]_1z ) * [v]_1z '
             ')'
             '+ 0.6666666666666666_wp*visc* ( ' + divops+' )*( ' + divops+' ) '} 

SLaplace['et'] = SLaplace['et'] + '- kappa* ( [T]_2xx + [T]_2yy + [T]_2zz )'                          