import numpy as np
import dnamiF
import sys

import dnami_io

from dnami_mpi import type_mpi

# =============================================================================
# TOOL BOX
# =============================================================================

def cst(x):
	from rhsinfo import wp
	if wp == 'float32': x = np.float32(x)
	if wp == 'float64': x = np.float64(x)
	return x

# =============================================================================
# INIT + MEM ALLOC
# =============================================================================

def create_tree():
	dtree = {}

	dtree['stats'] = None
	dtree['libs']  = {'fort':
							   {'integers': None, 'floats': None, 'data': None}
					 ,'cache blocking': None}
	dtree['grid']  = {'size' :
							   {'nxgb': None, 'nygb': None, 'nzgb': None}
					 ,'geom' :
							   {'Lx'  : None, 'Ly'  : None, 'Lz'  : None
							   ,'dx'  : None, 'dy'  : None, 'dz'  : None
							   ,'x'   : None, 'y'   : None, 'z'   : None
							   ,'xloc': None, 'yloc': None, 'zloc': None}}							   					 		   
	dtree['eqns']  = {'qvec' : 
							   {'nvars': None, 'solved': None, 'stored': None, 'views': None}
					 ,'coeff': None
					 ,'time' : None
					 ,'ndim' : None}

	dtree['misc']  = {'verbose': None, 'working precision': None}
	dtree['mpi']   = {'split': 
							   {'nxpr': None, 'nypr': None, 'nzpr': None}
					 ,'dMpi' : None}		   	
	dtree['num']   = {'hlo'  : None
					 ,'deriv': 
							   {'order': None, 'stencil': None, 'hlo': None} 
					 ,'filtr': 
				     	       {'order': None, 'stencil': None, 'hlo': None,'eps': None}
					 ,'tint' : 
					           {'tstep': None, 'cfl': None, 'itn': None}}

	dtree['bc']    = {'wall': 
							   {'isoT': None, 'zeroQ': None, 'slip': None}}	

	dtree['usr']   = None
	dtree['ios']   = None
	
	from rhsinfo import dim, stencil, order, coefficients, varname, wp,hlo_rhs

	dtree['eqns']['qvec']['solved'] = []
	for v in varname:
		dtree['eqns']['qvec']['solved'].append([v,varname[v]])

	dtree['eqns']['coeff'] = []
	for v in coefficients:
		dtree['eqns']['coeff'].append([v,coefficients[v]])	

	dtree['eqns']['qvec']['nvars']   = len(dtree['eqns']['qvec']['solved'])	
	dtree['num']['deriv']['stencil'] = stencil
	dtree['num']['deriv']['hlo']     = hlo_rhs #int((stencil-1)/2)
	dtree['num']['deriv']['order']   = order
	
	# if dtree['num']['filtr']['hlo'] != None:
	# 	dtree['num']['hlo'] = max(dtree['num']['deriv']['hlo'],dtree['num']['filtr']['hlo'])
	# else:
	# 	dtree['num']['hlo'] = dtree['num']['deriv']['hlo']	
                
	dtree['num']['hlo'] = hlo_rhs			
	
	dtree['eqns']['ndim'] = dim
	dtree['misc']['working precision'] = wp
	dtree['misc']['verbose'] = True

	dtree['libs']['cache blocking'] = [256,2,3]

	return dtree

def start_mpi(tree):
	dMpi = type_mpi(tree)
	tree['mpi']['dMpi'] = dMpi
	return tree

def allocate(tree):	

	wp   = tree['misc']['working precision']	
	dim  = tree['eqns']['ndim']
	nvar = tree['eqns']['qvec']['nvars']
	hlod = tree['num']['deriv']['hlo']
	hlof = tree['num']['filtr']['hlo']

	hlo  = tree['num']['hlo']  
	print("hlo dnami py",hlo)
	nx   = tree['mpi']['dMpi'].nx 
	ny   = tree['mpi']['dMpi'].ny 
	nz   = tree['mpi']['dMpi'].nz 

	variables = []
	if tree['eqns']['qvec']['solved']:
		for v in tree['eqns']['qvec']['solved']:
			variables.append(v[1])
	
	coeff = []
	if tree['eqns']['coeff']:
		for v in tree['eqns']['coeff']:
			coeff.append(v[1])

	sizex = nx + 2*hlo
	
	if(ny == 1):
		sizey = 1
	else:		
		sizey = ny + 2*hlo
	
	if(nz == 1) : 
		sizez = 1
	else:	
		sizez = nz + 2*hlo
	

	ndimpt  = sizex*sizey*sizez
	ndimtot = ndimpt*nvar

	# Integers parameters to be passed to the fortran layer
	intparam = np.empty(shape=11+nvar, dtype=np.int32,   order='F')     
	intparam[0]    = hlo
	intparam[1]    = nx
	intparam[2]    = ny
	intparam[3]    = nz
	intparam[4]    = nvar
	intparam[5]    = ndimtot
	intparam[6]    = 3                        # for RK sub steps in Python
	intparam[7]    = ndimpt
	intparam[8:8+3]    = tree['libs']['cache blocking']
	intparam[11:11+nvar] = variables[0:0+nvar]  # variables location (in q).
	
	if tree['eqns']['coeff']: 
		ncoef    = len(tree['eqns']['coeff'])
	else:
		ncoef = 0	

	# Floating point parameters to be passed to the Fortran layer (3 additional floats for the metrics, uniforme grid + 1 for dt  +1 for eps filter)
	fltparam = np.empty(shape=ncoef+5, dtype=wp, order='F')   

	fltparam[0] = cst(1.0)/tree['grid']['geom']['dx']
	if(dim>=2)   : fltparam[1] = cst(1.0)/tree['grid']['geom']['dy']
	if(dim == 3) : fltparam[2] = cst(1.0)/tree['grid']['geom']['dz']

	fltparam[3] = tree['num']['tint']['tstep']
	fltparam[4] = tree['num']['filtr']['eps']

	for i,v in enumerate(tree['eqns']['coeff']):
		fltparam[i+5] = v[1]

	# Floating point array (contiguous, aligned)
	data     = np.empty(shape=ndimtot*4, dtype=wp,order='F') # 4 -->  q,q1,q2 + rhs
	
	# Explicit view of data (only references here, no copy) 
	views = {}
	print("before views...")
	if dim == 3:
		views['q'] = data[0:ndimtot].reshape(sizex,sizey,sizez,nvar, order='F')
		for v in tree['eqns']['qvec']['solved']:
			views[v[0]] = data[(v[1]-1)*(ndimpt):(v[1])*ndimpt].view().reshape(sizex,sizey,sizez, order='F')
	elif dim == 2:
		views['q'] = data[0:ndimtot].view().reshape(sizex,sizey,nvar, order='F')
		for v in tree['eqns']['qvec']['solved']:
			views[v[0]] = data[(v[1]-1)*(ndimpt):(v[1])*ndimpt].view().reshape(sizex,sizey, order='F')	
	else:
		views['q'] =  data[0:ndimtot].view().reshape(sizex,nvar, order='F')
		for v in tree['eqns']['qvec']['solved']:
			views[v[0]] = data[(v[1]-1)*(ndimpt):(v[1])*ndimpt].view().reshape(sizex, order='F')				
	
	tree['libs']['fort']['integers'] = intparam
	tree['libs']['fort']['floats']   = fltparam
	tree['libs']['fort']['data']     = data
	tree['eqns']['qvec']['views']    = views

	print("Before init numa...")
	dnamiF.init(intparam,fltparam,data)	
	
	return tree

# =============================================================================
# GEOMETRY
# =============================================================================

def create_grid(tree):
	
	wp   = tree['misc']['working precision'] 
	ndim = tree['eqns']['ndim']
	grid = tree['grid']['size']
	geom = tree['grid']['geom']

	nxgb, nygb, nzgb  = grid['nxgb'], grid['nygb'], grid['nzgb'] 
	Lx  , Ly  , Lz    = geom['Lx']  , geom['Ly']  , geom['Lz'] 

	dmpi = tree['mpi']['dMpi']
	ibeg,jbeg,kbeg = dmpi.ibeg,dmpi.jbeg,dmpi.kbeg
	iend,jend,kend = dmpi.iend,dmpi.jend,dmpi.kend

	# create domain
	#                                   pt 1     2          n-1    n     period
	#                                   0                             L  |
	# full domain is [0,L] but careful: |--o--|--o--| ... |--o--|--o--|--V
	#                                       \___________________________/ 
	
	dx = Lx/cst(nxgb)
	x  = np.arange(dx/cst(2.),Lx,dx,dtype=wp)

	if nygb > 1:
		dy = Ly/cst(nygb)
		y  = np.arange(dy/cst(2.),Ly,dy,dtype=wp)
	else:
		Ly = cst(0.); y = []; dy = cst(0.)
	if nzgb > 1:
		dz = Lz/cst(nzgb)
		z  = np.arange(dz/cst(2.),Lz,dz,dtype=wp)
	else:
		Lz = cst(0.); z = []; dz = cst(0.)

	geom['dx'], geom['dy'], geom['dz'] = dx, dy, dz
	geom['x'] , geom['y'] , geom['z']  = x , y , z

	# global             iend ! ibeg                    iend  !  ibeg
	# pt#          n-1    n   !   1     2          n-1    n   !   1     2
	#        ... |--o--|--o--|!|--o--|--o--| ... |--o--|--o--|!|--o--|--o--| ...
	# w/ hlo     [<------------------------------------------------------->]
	# loc py ind    0     1      hlo                   hlo+n-1      n+2*hlo-1
	# glo py ind             ibeg+hlo-1               iend+hlo-1
	
	if ndim == 3:
		xloc = x[ibeg-1:iend] # without halos
		yloc = y[jbeg-1:jend]
		zloc = z[kbeg-1:kend]
		xx,yy,zz = np.meshgrid(xloc,yloc,zloc,sparse=False,indexing='ij')
		geom['xloc'],geom['yloc'], geom['zloc'] = xloc,yloc,zloc
	elif ndim == 2:
		xloc = x[ibeg-1:iend] # without halos
		yloc = y[jbeg-1:jend]
		xx,yy = np.meshgrid(xloc,yloc,sparse=False,indexing='ij')
		geom['xloc'],geom['yloc'] = xloc,yloc
	else:
		xloc = x[ibeg-1:iend] # without halos
		geom['xloc'] = xloc
	return tree
