from dnami import np, sys

# =============================================================================
# MESH
# =============================================================================

def write_grid(tree):
	wp   = tree['misc']['working precision']
	grid = tree['grid']
	nxgb, nygb, nzgb = grid['size']['nxgb'], grid['size']['nygb'], grid['size']['nzgb']
	Lx  , Ly  , Lz   = grid['geom']['Lx']  , grid['geom']['Ly']  , grid['geom']['Lz']
	x   , y   , z    = grid['geom']['x']   , grid['geom']['y']   , grid['geom']['z']
	# write grid to file
	if tree['mpi']['dMpi'].ioproc:
		head = np.empty(6,dtype=wp)
		head[:] = [nxgb,nygb,nzgb,Lx,Ly,Lz]
		with open('./out/axes.bin',"wb") as fh:
			np.concatenate((head,x,y,z)).tofile(fh)
			fh.closed

# =============================================================================
# MESSAGES
# =============================================================================

def hello_world(tree):
	dmpi = tree['mpi']['dMpi']
	grid = tree['grid']

	ioproc = dmpi.ioproc	
	iMpi   = dmpi.iMpi
	if iMpi : nprocs = dmpi.nprocs
	nxpr, nypr, nzpr  = tree['mpi']['split']['nxpr'], tree['mpi']['split']['nypr'], tree['mpi']['split']['nzpr']
	
	nxgb, nygb, nzgb  = grid['size']['nxgb'], grid['size']['nygb'], grid['size']['nzgb']
	nx  , ny  , nz    = dmpi.nx             , dmpi.ny             , dmpi.nz
	Lx  , Ly  , Lz    = grid['geom']['Lx']  , grid['geom']['Ly']  , grid['geom']['Lz']
	x   , y   , z     = grid['geom']['x']   , grid['geom']['y']   , grid['geom']['z']
	dx  , dy  , dz    = grid['geom']['dx']  , grid['geom']['dy']  , grid['geom']['dz']
	
	if ioproc:
		print('\033[1;40;94m'+'  /\     /\_   '+'\033[1;40;37m'+'  ||\  |  _       o'+'\033[1;40;31m'+'   /\      __         '+'\033[0m')	
		print('\033[1;40;94m'+'\/  \  _/   \  '+'\033[1;40;37m'+' _|| \ | /_\ |\/| |'+'\033[1;40;31m'+'  /  \    /  \        '+'\033[0m')
		print('\033[1;40;94m'+'     \/      \ '+'\033[1;40;37m'+'(_||  \|/   \|  | |'+'\033[1;40;31m'+' /    \__/    \__ v0.0'+'\033[0m')
		print('\033[1m'+'Geometry:'+'\033[0m')	
		print('-- grid size (nxgb,nygb,nzgb):',nxgb,nygb,nzgb)
		if iMpi:
			print('-- running with mpi')
			print('-- number of requested processes:',nxpr,'x',nypr,'x',nzpr,'=',nprocs)
		else:
			print('-- running with NO mpi')
		print('-- local grid size (nx,ny,nz):',nx,ny,nz)
		print('-- xmin/max,dx:',x[0],x[-1],dx)
		if nygb > 1: print('-- ymin/max,dy:',y[0],y[-1],dy)
		if nzgb > 1: print('-- zmin/max,dz:',z[0],z[-1],dz)
		sys.stdout.flush()

# =============================================================================
# RESTARTS
# =============================================================================

def write_restart(n,t,flag,tree):
	# file name switch (for live views)
	if flag == 0:
		fname = './restarts/restart_' + str(n).zfill(8)
	else:
		fname = './out/liv/restart.bin'
	
	# unpack useful tree data
	wp   = tree['misc']['working precision']
	dmpi = tree['mpi']['dMpi']
	grid = tree['grid']
	# nvar = tree['eqns']['qvec']['nvars']
	nvar = len(tree['eqns']['qvec']['solved'])
	hlo  = tree['num']['hlo']
	ndim = tree['eqns']['ndim']
	q    = tree['eqns']['qvec']['views']['q']

	ioproc = dmpi.ioproc
	iMpi   = dmpi.iMpi
	if iMpi : 
		nprocs = dmpi.nprocs
		MPIWP  = dmpi.MPIWP
		MPI    = dmpi.MPIlib

	nxgb, nygb, nzgb = grid['size']['nxgb'], grid['size']['nygb'], grid['size']['nzgb']
	nx  , ny  , nz   = dmpi.nx             , dmpi.ny             , dmpi.nz
	
	# header
	headsize = 7; head = np.empty(headsize,dtype=wp)
	head[:] = [headsize,n,t,nxgb,nygb,nzgb,nvar] # implicit type conversion
	# body

	dat = np.empty((nx,ny,nz,nvar),dtype=wp)
	if ndim == 3:
		dat = q[hlo:hlo+nx,hlo:hlo+ny,hlo:hlo+nz,0:nvar].copy()
	elif ndim == 2:
		dat = q[hlo:hlo+nx,hlo:hlo+ny,np.newaxis,0:nvar].copy()
	else:
		dat = q[hlo:hlo+nx,np.newaxis,np.newaxis,0:nvar].copy()
	if iMpi:
		header = MPIWP.Create_contiguous(headsize)
		header.Commit()	
		fh = MPI.File.Open(dmpi.comm_torus,fname,MPI.MODE_WRONLY|MPI.MODE_CREATE)
		fh.Set_view(0,MPIWP,header)
		if ioproc: fh.Write_at(0,head)
		header.Free()	
		subarray = MPIWP.Create_subarray((nxgb,nygb,nzgb,nvar),(nx,ny,nz,nvar),(dmpi.ibeg-1,dmpi.jbeg-1,dmpi.kbeg-1,0))
		subarray.Commit()
		disp = MPIWP.Get_size()*headsize
		fh.Set_view(disp,MPIWP,subarray)
		fh.Write_all(dat)
		subarray.Free()
		fh.Close()
	else:
		with open(fname,"wb") as fh:
			np.concatenate((head,np.reshape(dat,(nx*ny*nz*nvar)))).tofile(fh)
		fh.closed

def read_restart(tree):
	# unpack useful tree data
	wp   = tree['misc']['working precision']
	dmpi = tree['mpi']['dMpi']
	grid = tree['grid']
	# nvar = tree['eqns']['qvec']['nvars']
	nvar = len(tree['eqns']['qvec']['solved'])
	hlo  = tree['num']['hlo']
	ndim = tree['eqns']['ndim']
	q    = tree['eqns']['qvec']['views']['q']

	ioproc = dmpi.ioproc
	iMpi   = dmpi.iMpi
	if iMpi : 
		nprocs = dmpi.nprocs
		MPIWP  = dmpi.MPIWP
		MPI    = dmpi.MPIlib

	nxgb, nygb, nzgb = grid['size']['nxgb'], grid['size']['nygb'], grid['size']['nzgb']
	nx  , ny  , nz   = dmpi.nx             , dmpi.ny             , dmpi.nz

	# read in restart
	fname = 'restart.bin'
	headsize = 7
	if iMpi:
		head = np.empty(headsize,dtype=wp)
		dat  = np.empty((nx,ny,nz,nvar),dtype=wp)
		header = MPIWP.Create_contiguous(headsize)
		header.Commit()	
		fh = MPI.File.Open(dmpi.comm_torus,fname,MPI.MODE_RDONLY)
		fh.Set_view(0,MPIWP,header)
		fh.Read_at(0,head)
		header.Free()	
		subarray = MPIWP.Create_subarray((nxgb,nygb,nzgb,nvar),(nx,ny,nz,nvar),(dmpi.ibeg-1,dmpi.jbeg-1,dmpi.kbeg-1,0))
		subarray.Commit()
		disp = MPIWP.Get_size()*headsize
		fh.Set_view(disp,MPIWP,subarray)
		fh.Read_all(dat)
		subarray.Free()
		fh.Close()
	else:
		with open(fname,"rb") as fh:
			head = np.fromfile(fh,dtype=wp,count=headsize)
			dat  = np.fromfile(fh,dtype=wp,count=nx*ny*nz*nvar)
			dat  = np.reshape(dat,(nx,ny,nz,nvar))
		fh.closed
	
	# run some checks & print some info to terminal:
	if ioproc:
		print('\033[1;32m'+'================================'+'\033[0m')
		print('\033[1;32m'+'RESTARTING FROM RESTART.BIN FILE'+'\033[0m')
		print('\033[1;32m'+'================================'+'\033[0m')
		print('\033[1m'+'From header:'+'\033[0m')
		print('n,time:',int(head[1]),',',head[2])
		print('nxgb,nygb,nzgb,nvar:',int(head[3]),',',int(head[4]),',',int(head[5]),',',int(head[6]))
		if nxgb != head[3]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nxgb does not match that of restart.bin'); sys.exit()
		if nygb != head[4]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nygb does not match that of restart.bin'); sys.exit()
		if nzgb != head[5]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nzgb does not match that of restart.bin'); sys.exit()
		if nvar != head[6]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nvar does not match that of restart.bin'); sys.exit()
		print('We are good to go!')
	
	# set iteration number and time:
	tree['num']['tint']['itn'] = int(head[1])
	tree['eqns']['time']       = head[2]
	
	# set fields:
	if ndim == 3:
		q[hlo:hlo+nx,hlo:hlo+ny,hlo:hlo+nz,0:nvar] = dat.copy()
	elif ndim == 2:
		q[hlo:hlo+nx,hlo:hlo+ny,           0:nvar] = dat.reshape(nx,ny,nvar).copy()
	elif ndim == 1:
		q[hlo:hlo+nx,                      0:nvar] = dat.reshape(nx,nvar).copy()
	else:
		if ioproc: print('[error] (read_restart) ndim (',ndim,') should be 1, 2 or 3')
		sys.exit()
	return tree

'''
if iMpi:
	if wp == 'float64': MPIWP = MPI.DOUBLE
	if wp == 'float32': MPIWP = MPI.FLOAT

def write_restart(n,t,f,flag):
	# write bare minimum to restart a run from the instant of the function call
	if flag == 0:	
		fname = './restarts/restart_' + str(n).zfill(8)
	else:
		fname = './out/liv/restart.bin'
	# header
	headsize = 7
	head    = np.empty(headsize,dtype=wp)
	head[:] = [headsize,n,t,nxgb,nygb,nzgb,nvar] # implicit type conversion
	# body
	dat = np.empty((nx,ny,nz,nvar),dtype=wp)
	if ndim == 3:
		dat = f[hlo:hlo+nx,hlo:hlo+ny,hlo:hlo+nz,np.newaxis].copy()
	elif ndim == 2:
		dat = f[hlo:hlo+nx,hlo:hlo+ny,np.newaxis,np.newaxis].copy()
	else:
		dat = f[hlo:hlo+nx,np.newaxis,np.newaxis,np.newaxis].copy()	
	if iMpi:
		header = MPIWP.Create_contiguous(headsize)
		header.Commit()	
		fh = MPI.File.Open(comm_torus,fname,MPI.MODE_WRONLY|MPI.MODE_CREATE)
		fh.Set_view(0,MPIWP,header)
		if ioproc: fh.Write_at(0,head)
		header.Free()	
		subarray = MPIWP.Create_subarray((nxgb,nygb,nzgb,nvar),(nx,ny,nz,nvar),(ibeg-1,jbeg-1,kbeg-1,0))
		subarray.Commit()
		disp = MPIWP.Get_size()*headsize
		fh.Set_view(disp,MPIWP,subarray)
		fh.Write_all(dat)
		subarray.Free()
		fh.Close()
	else:
		with open(fname,"wb") as fh:
			np.concatenate((head,np.reshape(dat,(nx*ny*nz*nvar)))).tofile(fh)
		fh.closed
	return

def read_restart():
	# read in restart
	fname = 'restart.bin'
	headsize = 7
	if iMpi:
		head = np.empty(headsize,dtype=wp)
		dat  = np.empty((nx,ny,nz,nvar),dtype=wp)
		header = MPIWP.Create_contiguous(headsize)
		header.Commit()	
		fh = MPI.File.Open(comm_torus,fname,MPI.MODE_RDONLY)
		fh.Set_view(0,MPIWP,header)
		fh.Read_at(0,head)
		header.Free()	
		subarray = MPIWP.Create_subarray((nxgb,nygb,nzgb,nvar),(nx,ny,nz,nvar),(ibeg-1,jbeg-1,kbeg-1,0))
		subarray.Commit()
		disp = MPIWP.Get_size()*headsize
		fh.Set_view(disp,MPIWP,subarray)
		fh.Read_all(dat)
		subarray.Free()
		fh.Close()
	else:
		with open(fname,"rb") as fh:
			head = np.fromfile(fh,dtype=wp,count=headsize)
			dat  = np.fromfile(fh,dtype=wp,count=nx*ny*nz*nvar)
			dat  = np.reshape(dat,(nx,ny,nz,nvar))
		fh.closed
	# run some checks & print some info to terminal:
	if ioproc:
		print('\033[1;32m'+'================================'+'\033[0m')
		print('\033[1;32m'+'RESTARTING FROM RESTART.BIN FILE'+'\033[0m')
		print('\033[1;32m'+'================================'+'\033[0m')
		print('\033[1m'+'From header:'+'\033[0m')
		print('n,time:',int(head[1]),',',head[2])
		print('nxgb,nygb,nzgb,nvar:',int(head[3]),',',int(head[4]),',',int(head[5]),',',int(head[6]))
		if nxgb != head[3]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nxgb does not match that of restart.bin'); sys.exit()
		if nygb != head[4]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nygb does not match that of restart.bin'); sys.exit()
		if nzgb != head[5]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nzgb does not match that of restart.bin'); sys.exit()
		if nvar != head[6]: print('\033[1;40;31m'+'[error]'+'\033[0m'+' nvar does not match that of restart.bin'); sys.exit()
		print('We are good to go!')
	# set iteration number and time:
	n  = int(head[1])
	ti = head[2]
	# set fields:
	if ndim == 3:
		f[hlo:hlo+nx,hlo:hlo+ny,hlo:hlo+nz,np.newaxis] = dat.copy()
	elif ndim == 2:
		f[hlo:hlo+nx,hlo:hlo+ny,np.newaxis,np.newaxis] = dat.copy()
	else:
		f[hlo:hlo+nx,np.newaxis,np.newaxis,np.newaxis] = dat.copy()
	return
'''
