import math as m
import dnami as dn
from dnami import np, sys # re-import non dnami modules
import os

dtree = dn.create_tree()

Ma    = dn.cst(0.1)
# Re    = dn.cst(1600.0)
Re    = dn.cst(10053.0)
Pr    = dn.cst(0.71)
gamma = dn.cst(1.4)
dimPcr = False
if dimPcr:
	Cv = dn.cst(1.0/(gamma-1.0))
else:
	Cv = dn.cst(1.0/(Ma**2*gamma*(gamma-1.0)))

dtree['eqns']['coeff'][0][1] = dn.cst(1.0/Re)
dtree['eqns']['coeff'][1][1] = dn.cst(1.0/( (gamma-1.0)*Ma**2*Re*Pr ))
dtree['eqns']['coeff'][2][1] = dn.cst(gamma-1.)
dtree['eqns']['coeff'][3][1] = dn.cst(1.0/Cv)


numerics = dtree['num']
grid     = dtree['grid']['size']
geom     = dtree['grid']['geom']
mpi      = dtree['mpi']['split']

grid['nxgb'] = 128
grid['nygb'] = 128
grid['nzgb'] = 128

geom['Lx'] = dn.cst(1.0) 
geom['Ly'] = dn.cst(1.0)
geom['Lz'] = dn.cst(1.0)

mpi['nxpr'] = 1
mpi['nypr'] = 1
mpi['nzpr'] = 1

dtree = dn.start_mpi(dtree)

dMpi = dtree['mpi']['dMpi']

nx  = dMpi.nx
ny  = dMpi.ny
nz  = dMpi.nz


dtree = dn.create_grid(dtree)
dn.dnami_io.write_grid(dtree)
dn.dnami_io.hello_world(dtree)

xloc, yloc, zloc  = geom['xloc'], geom['yloc'], geom['zloc']
Lx  , Ly  , Lz    = geom['Lx']  , geom['Ly']  , geom['Lz']
dx  , dy  , dz    = geom['dx']  , geom['dy']  , geom['dz']

#=====================================================================================================================
#
#                                                ALLOCATE DATA
#
#=====================================================================================================================

numerics['tint']['tstep'] = dn.cst(0.001)
dt = numerics['tint']['tstep']

numerics['filtr']['eps'] = dn.cst(0.6)

dtree['libs']['cache blocking'] = []
for arg in sys.argv[1:]:
	print(int(arg))
	dtree['libs']['cache blocking'].append(int(arg))

dtree = dn.allocate(dtree)

rh = dtree['eqns']['qvec']['views']['rho']
ux = dtree['eqns']['qvec']['views']['u']
uy = dtree['eqns']['qvec']['views']['v']
uz = dtree['eqns']['qvec']['views']['w']
et = dtree['eqns']['qvec']['views']['et']

q  = dtree['eqns']['qvec']['views']['q'] 

#=====================================================================================================================
#
#                                                Initialisation 
#
#=====================================================================================================================

hlo  = numerics['hlo']

#init thermo
T0   = dn.cst(1.0)
P0   = dn.cst(1.0)
Rho0 = P0/T0

#pulse shape
eps = dn.cst(0.3)
R0=dn.cst(0.1*Lx)
R0=R0/(dn.cst(2.0)*np.sqrt(dn.cst(2.0)*np.log(dn.cst(2.0))))

#init field

# rh[hlo:nx+hlo,hlo:ny+hlo] = Rho0#*(dn.cst(1.0) + eps*np.asarray(np.exp(-((xloc[:, np.newaxis]-dn.cst(0.5)*Lx)**2/(dn.cst(2.0)*R0**2)
# 																	   # +(yloc[:]-dn.cst(0.5)*Ly)**2/(dn.cst(2.0)*R0**2))),dtype=np.float64, order='Fortran'))#eps*np.asarray(np.sin(2.0*m.pi*(xloc[:, np.newaxis]+yloc[:])/0.5),dtype=np.float64, order='Fortran')

# ux[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(0.0)#eps*np.asarray(np.exp(-((xloc[:, np.newaxis]-dn.cst(0.5)*Lx)**2/(dn.cst(2.0)*R0**2)+(yloc[:]-dn.cst(0.5)*Ly)**2/(dn.cst(2.0)*R0**2))),dtype=np.float64, order='Fortran')
# uy[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(0.0)

# p = P0*(dn.cst(1.0) + eps*np.asarray(np.exp(-((xloc[:, np.newaxis]-dn.cst(0.5)*Lx)**2/(dn.cst(2.0)*R0**2)
# 																	   +(yloc[:]-dn.cst(0.5)*Ly)**2/(dn.cst(2.0)*R0**2))),dtype=np.float64, order='Fortran'))#eps*np.asarray(np.sin(2.0*m.pi*(xloc[:, np.newaxis]+yloc[:])/0.5),dtype=np.float64, order='Fortran')

# et[hlo:nx+hlo,hlo:ny+hlo] = p/rh[hlo:nx+hlo,hlo:ny+hlo]*dn.cst(1./(gamma-1.)) + dn.cst(0.5)*(ux[hlo:nx+hlo,hlo:ny+hlo]*ux[hlo:nx+hlo,hlo:ny+hlo]
# 						+ uy[hlo:nx+hlo,hlo:ny+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo])	


# Init TGV

rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = Rho0

ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (np.sin(dn.cst(2.0)*m.pi*xloc[:         , np.newaxis, np.newaxis])
									   *np.cos(dn.cst(2.0)*m.pi*yloc[np.newaxis, :         , np.newaxis])
									   *np.cos(dn.cst(2.0)*m.pi*zloc[np.newaxis, np.newaxis, :         ]))

uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (-np.cos(dn.cst(2.0)*m.pi*xloc[:         , np.newaxis, np.newaxis])
										*np.sin(dn.cst(2.0)*m.pi*yloc[np.newaxis, :         , np.newaxis])
										*np.cos(dn.cst(2.0)*m.pi*zloc[np.newaxis, np.newaxis, :         ]))

uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(0.0)

p =(P0 + dn.cst(Rho0/16.0)*(np.cos( dn.cst(2.0) * (dn.cst(2.0)*m.pi*xloc[:         , np.newaxis, np.newaxis]) )
						   +np.cos( dn.cst(2.0) * (dn.cst(2.0)*m.pi*yloc[np.newaxis, :         , np.newaxis]) ))
						  *(np.cos( dn.cst(2.0) * (dn.cst(2.0)*m.pi*zloc[np.newaxis, np.newaxis, :         ]) ) + dn.cst(2.0)))


et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (  p/rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*dn.cst(1./(gamma-1.)) 
						 + dn.cst(0.5)*(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]
						 			   +uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]
						 			   +uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))

# 						+ uy[hlo:nx+hlo,hlo:ny+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo])	


del p

dn.dnami_io.write_restart(0,dn.cst(0.),1,dtree)
dn.dnami_io.write_restart(0,dn.cst(0.),0,dtree)


# rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (dMpi.procid+1)
# print("I am proc:", dMpi.procid+1," rho  before swap is : "+str(rh.shape)+"\n", rh[:,0,:])
# sys.stdout.flush()

dMpi.swap(q,hlo,dtree)

# print("I am proc:", dMpi.procid+1," rho  after swap is : "+str(rh.shape)+"\n", rh[:,0,:])
# sys.stdout.flush()

# sys.exit()
#=====================================================================================================================
#
#                                                dNami time loop 
#
#=====================================================================================================================


ti = dn.cst(0.0)

intparam,fltparam,data = (dtree['libs']['fort']['integers'],
						  dtree['libs']['fort']['floats'],
						  dtree['libs']['fort']['data'])


# dtree = dn.dnami_io.read_restart(dtree)

nitmax = 20

c = np.sqrt(gamma*P0/rh[hlo:nx+hlo,hlo:ny+hlo])

print("CFL_x is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)
print("CFL_y is :",dt*np.max(uy[hlo:nx+hlo,hlo:ny+hlo]+c)/dy)
print("CFL_z is :",dt*np.max(uz[hlo:nx+hlo,hlo:ny+hlo]+c)/dz)

# sys.exit()

mod_filter = 10

from timeit import default_timer as timer
t0 = timer()

for n in range(0,nitmax+1):
	ti = ti + dt
	for nrk in range(1,4):
		intparam[6] = nrk	
		dn.dnamiF.time(intparam,fltparam,data)	
		dMpi.swap(q,hlo,dtree)

	if np.mod(n,mod_filter) == 0:
		
		dn.dnamiF.filter(1,intparam,fltparam,data)

		dMpi.swapYc(q,hlo,dtree)
		dn.dnamiF.filter(2,intparam,fltparam,data)

		dMpi.swapZc(q,hlo,dtree)
		dn.dnamiF.filter(3,intparam,fltparam,data)


	# if np.mod(n,100) == 0 : 
		# if dMpi.ioproc == True: 
			# print('time step, time : ',n,ti)
			# print('min max rho',np.min(rh),np.max(rh))
			# c = np.sqrt(gamma*P0/rh[hlo:nx+hlo,hlo:ny+hlo])

			# print("CFL_x is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)
			# print("CFL_y is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)
			# print("CFL_z is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)

			# sys.stdout.flush()

		# # --- for live view ---
		# liveViewNow = False
		# if dMpi.ioproc == True:
		# 	if os.path.exists('./out/liv/snap'):
		# 		snap_time = os.path.getmtime('./out/liv/snap')
		# 		if os.path.exists('./out/liv/restart.bin'):
		# 			data_time = os.path.getmtime('./out/liv/restart.bin')
		# 			if snap_time > data_time: liveViewNow = True
		# 		else:
		# 			liveViewNow = True
		# if dMpi.iMpi: liveViewNow = dMpi.comm_torus.bcast(liveViewNow,root=0)
		# if liveViewNow: dn.dnami_io.write_restart(n,ti,1,dtree)
		#--- end of for live view ---

t1 = timer()
tfile = open('timeblck.dat','a')
tfile.write(str(dtree['libs']['cache blocking'][0])+'  '+
			str(dtree['libs']['cache blocking'][1])+'  '+
			str(dtree['libs']['cache blocking'][2])+'  '+
			str(t1-t0)+'\n')
dn.dnami_io.write_restart(n,ti,0,dtree)
