import math as m
import dnami as dn
from dnami import np, sys # re-import non dnami modules
import os

dtree = dn.create_tree()

Ma    = dn.cst(0.1)
Re    = dn.cst(1600.0)
# Re    = dn.cst(10053.0)
Pr    = dn.cst(0.71)
gamma = dn.cst(1.4)
dimPcr = False
if dimPcr:
	Cv = dn.cst(1.0/(gamma-1.0))
else:
	Cv = dn.cst(1.0/(Ma**2*gamma*(gamma-1.0)))

dtree['eqns']['coeff'][0][1] = dn.cst(1.0/Re)
dtree['eqns']['coeff'][1][1] = dn.cst(1.0/( (gamma-1.0)*Ma**2*Re*Pr ))
dtree['eqns']['coeff'][2][1] = dn.cst(gamma-1.)
dtree['eqns']['coeff'][3][1] = dn.cst(1.0/Cv)


numerics = dtree['num']
grid     = dtree['grid']['size']
geom     = dtree['grid']['geom']
mpi      = dtree['mpi']['split']

grid['nxgb'] = 64
grid['nygb'] = 64
grid['nzgb'] = 64

# geom['Lx'] = dn.cst(1.0) 
# geom['Ly'] = dn.cst(1.0)
# geom['Lz'] = dn.cst(1.0)

geom['Lx'] = dn.cst(2.0*m.pi) 
geom['Ly'] = dn.cst(2.0*m.pi)
geom['Lz'] = dn.cst(2.0*m.pi)


mpi['nxpr'] = 1
mpi['nypr'] = 1
mpi['nzpr'] = 2

dtree = dn.start_mpi(dtree)

dMpi = dtree['mpi']['dMpi']

nx  = dMpi.nx
ny  = dMpi.ny
nz  = dMpi.nz


dtree = dn.create_grid(dtree)
dn.dnami_io.write_grid(dtree)
dn.dnami_io.hello_world(dtree)

xloc, yloc, zloc  = geom['xloc'], geom['yloc'], geom['zloc']
Lx  , Ly  , Lz    = geom['Lx']  , geom['Ly']  , geom['Lz']
dx  , dy  , dz    = geom['dx']  , geom['dy']  , geom['dz']

#=====================================================================================================================
#
#                                                ALLOCATE DATA
#
#=====================================================================================================================

numerics['tint']['tstep'] = dn.cst(0.0015)
dt = numerics['tint']['tstep']

numerics['filtr']['eps'] = dn.cst(0.1)

# dtree['libs']['cache blocking'] = []
# for arg in sys.argv[1:]:
# 	print(int(arg))
# 	dtree['libs']['cache blocking'].append(int(arg))

dtree = dn.allocate(dtree)

rh = dtree['eqns']['qvec']['views']['rho']
ux = dtree['eqns']['qvec']['views']['u']
uy = dtree['eqns']['qvec']['views']['v']
uz = dtree['eqns']['qvec']['views']['w']
et = dtree['eqns']['qvec']['views']['et']

q  = dtree['eqns']['qvec']['views']['q'] 

#=====================================================================================================================
#
#                                                Initialisation 
#
#=====================================================================================================================

hlo  = numerics['hlo']

#init thermo
T0   = dn.cst(1.0)
P0   = dn.cst(1.0)/(Ma**2*gamma)
Rho0 = dn.cst(1.0)#P0/T0*Ma**2*gamma

# c = np.sqrt(gamma*p/rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo])

# U0 = Ma

#pulse shape
# eps = dn.cst(0.3)
# R0=dn.cst(0.1*Lx)
# R0=R0/(dn.cst(2.0)*np.sqrt(dn.cst(2.0)*np.log(dn.cst(2.0))))

#init field

# rh[hlo:nx+hlo,hlo:ny+hlo] = Rho0#*(dn.cst(1.0) + eps*np.asarray(np.exp(-((xloc[:, np.newaxis]-dn.cst(0.5)*Lx)**2/(dn.cst(2.0)*R0**2)
# 																	   # +(yloc[:]-dn.cst(0.5)*Ly)**2/(dn.cst(2.0)*R0**2))),dtype=np.float64, order='Fortran'))#eps*np.asarray(np.sin(2.0*m.pi*(xloc[:, np.newaxis]+yloc[:])/0.5),dtype=np.float64, order='Fortran')

# ux[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(0.0)#eps*np.asarray(np.exp(-((xloc[:, np.newaxis]-dn.cst(0.5)*Lx)**2/(dn.cst(2.0)*R0**2)+(yloc[:]-dn.cst(0.5)*Ly)**2/(dn.cst(2.0)*R0**2))),dtype=np.float64, order='Fortran')
# uy[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(0.0)

# p = P0*(dn.cst(1.0) + eps*np.asarray(np.exp(-((xloc[:, np.newaxis]-dn.cst(0.5)*Lx)**2/(dn.cst(2.0)*R0**2)
# 																	   +(yloc[:]-dn.cst(0.5)*Ly)**2/(dn.cst(2.0)*R0**2))),dtype=np.float64, order='Fortran'))#eps*np.asarray(np.sin(2.0*m.pi*(xloc[:, np.newaxis]+yloc[:])/0.5),dtype=np.float64, order='Fortran')

# et[hlo:nx+hlo,hlo:ny+hlo] = p/rh[hlo:nx+hlo,hlo:ny+hlo]*dn.cst(1./(gamma-1.)) + dn.cst(0.5)*(ux[hlo:nx+hlo,hlo:ny+hlo]*ux[hlo:nx+hlo,hlo:ny+hlo]
# 						+ uy[hlo:nx+hlo,hlo:ny+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo])	


# Init TGV

# rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = Rho0

# ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (np.sin(dn.cst(2.0)*m.pi*xloc[:         , np.newaxis, np.newaxis])
# 									   *np.cos(dn.cst(2.0)*m.pi*yloc[np.newaxis, :         , np.newaxis])
# 									   *np.cos(dn.cst(2.0)*m.pi*zloc[np.newaxis, np.newaxis, :         ]))

# uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (-np.cos(dn.cst(2.0)*m.pi*xloc[:         , np.newaxis, np.newaxis])
# 										*np.sin(dn.cst(2.0)*m.pi*yloc[np.newaxis, :         , np.newaxis])
# 										*np.cos(dn.cst(2.0)*m.pi*zloc[np.newaxis, np.newaxis, :         ]))

# uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(0.0)

# p =(P0 + dn.cst(Rho0/16.0)*(np.cos( dn.cst(2.0) * (dn.cst(2.0)*m.pi*xloc[:         , np.newaxis, np.newaxis]) )
# 						   +np.cos( dn.cst(2.0) * (dn.cst(2.0)*m.pi*yloc[np.newaxis, :         , np.newaxis]) ))
# 						  *(np.cos( dn.cst(2.0) * (dn.cst(2.0)*m.pi*zloc[np.newaxis, np.newaxis, :         ]) ) + dn.cst(2.0)))


# et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (  p/rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*dn.cst(1./(gamma-1.)) 
# 						 + dn.cst(0.5)*(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]
# 						 			   +uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]
# 						 			   +uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))

rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = Rho0

ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (np.sin(xloc[:         , np.newaxis, np.newaxis])
									   *np.cos(yloc[np.newaxis, :         , np.newaxis])
									   *np.cos(zloc[np.newaxis, np.newaxis, :         ]))

uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (-np.cos(xloc[:         , np.newaxis, np.newaxis])
										*np.sin(yloc[np.newaxis, :         , np.newaxis])
										*np.cos(zloc[np.newaxis, np.newaxis, :         ]))

uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(0.0)

p =(P0 + dn.cst(Rho0/16.0)*(np.cos( dn.cst(2.0) * (xloc[:         , np.newaxis, np.newaxis]) )
						   +np.cos( dn.cst(2.0) * (yloc[np.newaxis, :         , np.newaxis]) ))
						  *(np.cos( dn.cst(2.0) * (zloc[np.newaxis, np.newaxis, :         ]) ) + dn.cst(2.0)))


et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (  p/rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*dn.cst(1./(gamma-1.)) 
						 + dn.cst(0.5)*(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]
						 			   +uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]
						 			   +uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]*uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))

						# + uy[hlo:nx+hlo,hlo:ny+hlo]*uy[hlo:nx+hlo,hlo:ny+hlo])	


# del p


dn.dnami_io.write_restart(0,dn.cst(0.),1,dtree)
dn.dnami_io.write_restart(0,dn.cst(0.),0,dtree)
# ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] =  (dMpi.procid)
# for k in range(hlo,nz+hlo):
# 	for j in range(hlo,ny+hlo):
# 		for i in range(hlo,2*hlo):
# 			ux[i,j,k]  = -(dMpi.procid)
dMpi.swap(q,hlo,dtree)

'''
#rh[:,:,:] = 0.0
rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = (dMpi.procid)
np.set_printoptions(threshold=sys.maxsize)
print("I am proc:", dMpi.procid," rho  before swap is : "+str(rh.shape)+"\n", rh[:,5,:])
sys.stdout.flush()


#rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(1.0)
#ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(2.0)
#uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(3.0)
#uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(4.0)
#et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] = dn.cst(5.0)

#rh[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(1.0)
#ux[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(2.0)
#uy[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(3.0)
#uz[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(4.0)
#et[hlo:nx+hlo,hlo:ny+hlo] = dn.cst(5.0)

#print("BEFORE SWAP")
#print("q is",rh.shape,rh)
#sys.stdout.flush()
#sys.exit()
print("!! !! hlo is ",hlo)
dMpi.swap(q,hlo,dtree)

print("I am proc:", dMpi.procid," rho  after swap is : "+str(rh.shape)+"\n", rh[:,5,:])
sys.stdout.flush()

sys.exit()
'''
#=====================================================================================================================
#
#                                                dNami time loop 
#
#=====================================================================================================================


ti = dn.cst(0.0)

intparam,fltparam,data = (dtree['libs']['fort']['integers'],
						  dtree['libs']['fort']['floats'],
						  dtree['libs']['fort']['data'])


# dtree = dn.dnami_io.read_restart(dtree)

nitmax = 14000


c = np.sqrt(gamma*p/rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo])

print(np.max(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo] / c[:,:,:]),np.max(c))


print("CFL_x is :",dt*np.max(np.abs(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]+c[:,:,:]))/dx)
print("CFL_y is :",dt*np.max(np.abs(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]+c[:,:,:]))/dy)
print("CFL_z is :",dt*np.max(np.abs(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]+c[:,:,:]))/dz)


print("rho min max :",np.min(rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
print("u   min max :",np.min(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
print("v   min max :",np.min(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
print("w   min max :",np.min(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))	
print("p   min max:",np.min(p),np.max(p))
print("et  min max:",np.min(et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))


#sys.exit()
# sys.exit()

mod_filter = 1
mod_output = 1000

from timeit import default_timer as timer
t0 = timer()

NOMP=int(os.environ['OMP_NUM_THREADS'])

for n in range(1,nitmax+1):
	ti = ti + dt
	# t00 = timer()
	for nrk in range(1,4):
		intparam[6] = nrk

		# dn.dnamiF.bc(0,intparam,fltparam,data)
		dMpi.swap(	q,hlo,dtree)
		dn.dnamiF.time(intparam,fltparam,data)	
		# dMpi.swap(	q,hlo,dtree)
	
	if np.mod(n,mod_output) == 0:
		if dMpi.ioproc:
			print("time step :",n,ti)
			sys.stdout.flush()
			print("rho min max :",np.min(rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
			print("u   min max :",np.min(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
			print("v   min max :",np.min(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
			print("w   min max :",np.min(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))				
			print("et  min max:",np.min(et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
		dn.dnami_io.write_restart(n,ti,0,dtree)	



		# t11 = timer()
	# print('min max rho',np.min(rh),np.max(rh))
	# sys.stdout.flush()
	# if np.mod(n,mod_filter) == 0:
	# 	# print("in filter")
	# 	# sys.exit()
	# 	dMpi.swapX(q,hlo,dtree)
	# 	# dMpi.swap(	q,hlo,dtree)
	# 	# dn.dnamiF.bc(1,intparam,fltparam,data)
	# 	dn.dnamiF.filter(1,intparam,fltparam,data)
		
	# 	# print("HLO in time loop is :",hlo)
	# 	# dMpi.swapX(q,hlo,dtree)
	# 	dMpi.swapYc(q,hlo,dtree)
	# 	# dMpi.swap(	q,hlo,dtree)
	# 	# dn.dnamiF.bc(2,intparam,fltparam,data)
	# 	dn.dnamiF.filter(2,intparam,fltparam,data)

	# 	dMpi.swapZc(q,hlo,dtree)
	# 	# dMpi.swap(	q,hlo,dtree)
	# 	# dn.dnamiF.bc(3,intparam,fltparam,data)
	# 	dn.dnamiF.filter(3,intparam,fltparam,data)
		
		# dMpi.swap(	q,hlo,dtree)
	# t000 = timer()
	# print('cpu (s/pts/3xOMP)',(t000-t00)/(grid['nxgb']*grid['nygb']*grid['nzgb']*3)*NOMP*mpi['nxpr']*mpi['nypr']*mpi['nzpr'])
	    
	# print("Time",(t000-t00)/(128**3),(t11-t000)/(128**3))
	# print('min max rho',np.min(rh),np.max(rh))
	# sys.stdout.flush()
	# if np.mod(n,100) == 0 : 
		# if dMpi.ioproc == True: 
			# print('time step, time : ',n,ti)
			# print('min max rho',np.min(rh),np.max(rh))
			# c = np.sqrt(gamma*P0/rh[hlo:nx+hlo,hlo:ny+hlo])

			# print("CFL_x is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)
			# print("CFL_y is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)
			# print("CFL_z is :",dt*np.max(ux[hlo:nx+hlo,hlo:ny+hlo]+c)/dx)

			# sys.stdout.flush()

		# # --- for live view ---
		# liveViewNow = False
		# if dMpi.ioproc == True:
		# 	if os.path.exists('./out/liv/snap'):
		# 		snap_time = os.path.getmtime('./out/liv/snap')
		# 		if os.path.exists('./out/liv/restart.bin'):
		# 			data_time = os.path.getmtime('./out/liv/restart.bin')
		# 			if snap_time > data_time: liveViewNow = True
		# 		else:
		# 			liveViewNow = True
		# if dMpi.iMpi: liveViewNow = dMpi.comm_torus.bcast(liveViewNow,root=0)
		# if liveViewNow: dn.dnami_io.write_restart(n,ti,1,dtree)
		#--- end of for live view ---

print("Tend* :",ti*2.0*m.pi)

t1 = timer()
NOMP=int(os.environ['OMP_NUM_THREADS'])

print('Nb. thread is: ',NOMP)

print("Time",(t1-t0)/(grid['nxgb']*grid['nygb']*grid['nzgb']*3*nitmax)*NOMP*mpi['nxpr']*mpi['nypr']*mpi['nzpr'],t1-t0)
tfile = open('timeblck.dat','a')
tfile.write(str(dtree['libs']['cache blocking'][0])+'  '+
			str(dtree['libs']['cache blocking'][1])+'  '+
			str(dtree['libs']['cache blocking'][2])+'  '+
			str(t1-t0)+'\n')

dn.dnami_io.write_restart(n,ti,0,dtree)


c = np.sqrt(gamma*p/rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo])

print("CFL_x end is :",dt*np.max(np.abs(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]+c[:,:,:]))/dx)
print("CFL_y end is :",dt*np.max(np.abs(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]+c[:,:,:]))/dy)
print("CFL_z end is :",dt*np.max(np.abs(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]+c[:,:,:]))/dz)

print("rho min max end :",np.min(rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(rh[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
print("u   min max end :",np.min(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(ux[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
print("v   min max end :",np.min(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(uy[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
print("w   min max end :",np.min(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(uz[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))	
print("p   min max end :",np.min(p),np.max(p))
print("et  min max end :",np.min(et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]),np.max(et[hlo:nx+hlo,hlo:ny+hlo,hlo:nz+hlo]))
